--[[
Name: FrameLib-3.0
Author(s): beerke  (Minor tweak by Nirek to load via Libstub instead of Acelib)
Website: -
Documentation: -
SVN: -
Description: A library to make creating frames easier

License: LGPL v2.1
]]

local MAJOR = "FrameLib-3.0"
local MINOR = "1"
local FL = LibStub:NewLibrary(MAJOR, MINOR)


FL.ANCHOR = {}
FL.ANCHOR.BOTTOM = "BOTTOM"
FL.ANCHOR.BOTTOMLEFT = "BOTTOMLEFT"
FL.ANCHOR.BOTTOMRIGHT = "BOTTOMRIGHT"
FL.ANCHOR.CENTER = "CENTER"
FL.ANCHOR.LEFT = "LEFT"
FL.ANCHOR.RIGHT = "RIGHT"
FL.ANCHOR.TOP = "TOP"
FL.ANCHOR.TOPLEFT = "TOPLEFT"
FL.ANCHOR.TOPRIGHT = "TOPRIGHT"

FL.STRATA = {}
FL.STRATA.TOOLTIP = "TOOLTIP"
FL.STRATA.FULLSCREEN_DIALOG = "FULLSCREEN_DIALOG"
FL.STRATA.FULLSCREEN = "FULLSCREEN"
FL.STRATA.DIALOG = "DIALOG"
FL.STRATA.HIGH = "HIGH"
FL.STRATA.MEDIUM = "MEDIUM"
FL.STRATA.LOW = "LOW"
FL.STRATA.BACKGROUND = "BACKGROUND"

FL.LAYER = {}
FL.LAYER.BACKGROUND = "BACKGROUND" 
FL.LAYER.BORDER = "BORDER"
FL.LAYER.ARTWORK = "ARTWORK"
FL.LAYER.OVERLAY = "OVERLAY"
FL.LAYER.HIGHLIGHT = "HIGHLIGHT"

FL.UIOBJECT = {}
FL.UIOBJECT.BUTTON = "Button"
FL.UIOBJECT.CHECKBUTTON = "CheckButton"
FL.UIOBJECT.EDITBOX = "EditBox"
FL.UIOBJECT.FRAME = "Frame"
FL.UIOBJECT.REGION = "Region"
FL.UIOBJECT.SCROLLFRAME = "ScrollFrame"
FL.UIOBJECT.SLIDER = "Slider"

FL.CLICK = {}
FL.CLICK.LBUTTONUP = "LeftButtonUp" 
FL.CLICK.RBUTTONUP = "RightButtonUp" 
FL.CLICK.MBUTTONUP = "MiddleButtonUp" 
FL.CLICK.FOURBUTTONUP = "Button4Up" 
FL.CLICK.FIVEBUTTONUP = "Button5Up" 
FL.CLICK.LBUTTONDOWN = "LeftButtonDown" 
FL.CLICK.RBUTTONDOWN = "RightButtonDown" 
FL.CLICK.MBUTTONDOWN = "MiddleButtonDown" 
FL.CLICK.FOURBUTTONDOWN = "Button4Down" 
FL.CLICK.FIVEBUTTONDOWN = "Button5Down"

function FL:CreateFrame(f)
--~ 	UIObject
--~ 		FontInstance
--~ 			Font
--~ 		Region
--~ 			Frame
--~ 				Button
--~ 					CheckButton
--~ 					LootButton
--~ 				Cooldown
--~ 				ColorSelect
--~ 				EditBox
--~ 				GameToolTip
--~ 				MessageFrame
--~ 				Minimap
--~ 				Model
--~ 					PlayerModel
--~ 						DressUpModel
--~ 						TabardModel
--~ 				ScrollFrame
--~ 				ScrollingMessageFrame
--~ 				SimpleHTML
--~ 				Slider
--~ 				StatusBar
--~ 			LayeredRegion
--~ 				Texture
--~ 				FontString

--~ Return the object if it allready exists, no need to recreate it
	if getglobal(f.name) then
		return getglobal(f.name)
	end
	
	local frame = CreateFrame(f.frame,f.name,f.parent,f.inherits)	
	
	if tonumber(f.alpha) then frame:SetAlpha(tonumber(f.alpha)) end
	
	if frame:IsObjectType(self.UIOBJECT.REGION) then
		if f.setpoint then self:SetPoint(frame,f.setpoint) end
		
		self:SetTooltip(frame,f.tt, f.ttanchor, f.tttype)
		
		if tonumber(f.width) then frame:SetWidth(tonumber(f.width)) end
		if tonumber(f.height) then frame:SetHeight(tonumber(f.height)) end
		
		if tonumber(f.scale) then frame:SetScale(tonumber(f.scale)) end

		if frame:IsObjectType(self.UIOBJECT.FRAME) then
			if f.strata ~= nil then frame:SetFrameStrata(f.strata) end
			if f.clamptoscreen ~= nil then frame:SetClampedToScreen(f.clamptoscreen) end
			if f.movable ~= nil then frame:SetMovable(f.movable) end
			if f.enablemouse ~= nil then frame:EnableMouse(f.enablemouse) end
			if f.registerfordrag ~= nil then frame:RegisterForDrag(f.registerfordrag) end
			if f.hitrectinsets then self:SetHitRectInsets(frame,f.hitrectinsets) end
			
			if f.value and f.value.id then frame:SetID(f.value.id) end
			
			if frame:IsObjectType(self.UIOBJECT.BUTTON) then
				if f.value and f.value.text ~= nil then frame:SetText(f.value.text) end
				
				if frame:IsObjectType(self.UIOBJECT.CHECKBUTTON) then
					if f.value and f.value.checked ~= nil then frame:SetChecked(f.value.checked) end
				end
			elseif frame:IsObjectType(self.UIOBJECT.EDITBOX) then
				if f.value and f.value.value ~= nil then frame:SetText(f.value.value) end
				if f.value and f.value.numeric ~= nil then frame:SetNumeric(f.value.numeric) end
			elseif frame:IsObjectType(self.UIOBJECT.SLIDER) then
				if f.value and f.value.min ~= nil and f.value.max ~= nil then frame:SetMinMaxValues(f.value.min,f.value.max) end
				if f.value and f.value.value ~= nil then frame:SetValue(f.value.value) end
				if f.value and f.value.step ~= nil then frame:SetValueStep(f.value.step) end
				
				if f.value and f.value.mintext ~= nil then getglobal(frame:GetName() .. "Low"):SetText(f.value.mintext) end
				if f.value and f.value.maxtext ~= nil then getglobal(frame:GetName() .. "High"):SetText(f.value.maxtext) end
				if f.value and f.value.valuetext ~= nil then getglobal(frame:GetName() .. "Text"):SetText(f.value.valuetext) end
			end
		end
	end
	
	return frame
end

function FL:CreateCheckButton(f)
	f.frame = self.UIOBJECT.CHECKBUTTON
	f.inherits = "UICheckButtonTemplate"
		
	return self:CreateFrame(f)
end

function FL:CreateEditBox(f)
	f.frame = self.UIOBJECT.EDITBOX
	f.inherits = "InputBoxTemplate"
	
	return self:CreateFrame(f)
end

function FL:CreateSlider(f)
	f.frame = self.UIOBJECT.SLIDER
	f.inherits = "OptionsSliderTemplate"
	
	return self:CreateFrame(f)
end

function FL:CreateScrollFrame(f)
	f.frame = self.UIOBJECT.SCROLLFRAME
	f.inherits = "FauxScrollFrameTemplate"
	
	return self:CreateFrame(f)
end

function FL:CreateFontString(f)
	if getglobal(f.name) then
		return getglobal(f.name)
	end
	
	local fs = f.parent:CreateFontString(f.name, f.layer, f.inherits)	
	if f.text ~= nil then fs:SetText(f.text) end
	if f.justify ~= nil then fs:SetJustifyH(f.justify) end
	
	if f.setpoint.x == "font" then f.setpoint.x = fs:GetStringWidth() end
	
	self:SetPoint(fs,f.setpoint)
	
	return fs
end

function FL:CreateTexture(f)
	local tex = getglobal(f.name)
	if not tex then
		tex = f.parent:CreateTexture(f.name,f.layer,f.inherits)
	end
	
--~ 	local tex = f.parent:CreateTexture(f.name,f.layer,f.inherits)
	
	if f.width ~= nil then tex:SetWidth(f.width) end
	if f.height ~= nil then tex:SetHeight(f.height) end
--~ 	if f.texture ~= nil then tex:SetTexture(f.texture) end
	
	tex:SetTexture(f.texture)
	
	self:SetPoint(tex,f.setpoint)
	
	return tex
end

--~ Creates a tooltip with event handler if tooltip is not nil, or not false
--~ tttype :
--~ - HyperLink
--~ - Text (default)
--~ ttanchor : FL.ANCHOR
function FL:SetTooltip(frame, tt, ttanchor, tttype)
 	if tt == nil or not tt then return end
	
	if not ttanchor then ttanchor = frame end
	
	frame.tooltip = tt
	
	if tttype == "Hyperlink" then
		frame:SetScript("OnEnter", function()
			if this.tooltip then
				GameTooltip:SetOwner(this, ttanchor)
				GameTooltip:SetHyperlink(this.tooltip)
				GameTooltip:Show()		
			end
		end)
	else
		frame:SetScript("OnEnter", function()
			if this.tooltip then
				GameTooltip:SetOwner(this, ttanchor)
				GameTooltip:SetText(this.tooltip)
				GameTooltip:Show()		
			end
		end)
	end
	
	frame:SetScript("OnLeave", function()
		GameTooltip:Hide()	
	end)
end

function FL:SetHitRectInsets(frame,insets)
	if insets and frame then
		if insets.top and insets.bottom and insets.left and insets.right then
			frame:SetHitRectInsets(insets.left,insets.right,insets.top,insets.bottom)
		end
	end
end

function FL:SetPoint(frame,p)
	if p and frame then
		if p.point ~= il then
			frame:ClearAllPoints()
			if p.frame~=nil and p.relpoint~=nil and p.x~=nil and p.y~=nil then
				frame:SetPoint(p.point,p.frame,p.relpoint,p.x,p.y)
			elseif p.frame~=nil and p.relpoint~=nil then
				frame:SetPoint(p.point,p.frame,p.relpoint)
			elseif p.frame~=nil then
				frame:SetPoint(p.point,p.frame)
			elseif p.relpoint~=nil and p.x~=nil and p.y~=nil then			
				frame:SetPoint(p.point,p.relpoint,p.x,p.y)
			elseif p.relpoint~=nil then
				frame:SetPoint(p.point,p.relpoint)
			elseif p.x~=nil and p.y~=nil then
				frame:SetPoint(p.point,p.x,p.y)
			else
				frame:SetPoint(p.point)
			end
		elseif p.frame ~= nil then
			frame:ClearAllPoints()
			frame:SetAllPoints(p.frame)
		end	
	end
end